@@ -1,20 +1,21 @@
-/****************************************************************************
- * NCSA HDF                                                                 *
- * Software Development Group                                               *
- * National Center for Supercomputing Applications                          *
- * University of Illinois at Urbana-Champaign                               *
- * 605 E. Springfield, Champaign IL 61820                                   *
- *                                                                          *
- * For conditions of distribution and use, see the accompanying             *
- * hdf/COPYING file.                                                        *
- *                                                                          *
- ****************************************************************************/
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * Copyright by The HDF Group.                                               *
+ * Copyright by the Board of Trustees of the University of Illinois.         *
+ * All rights reserved.                                                      *
+ *                                                                           *
+ * This file is part of HDF.  The full HDF copyright notice, including       *
+ * terms governing use, modification, and redistribution, is contained in    *
+ * the files COPYING and Copyright.html.  COPYING can be found at the root   *
+ * of the source code distribution tree; Copyright.html can be found at      *
+ * http://hdfgroup.org/products/hdf4/doc/Copyright.html.  If you do not have *
+ * access to either file, you may request a copy from help@hdfgroup.org.     *
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifdef RCSID
-static char RcsId[] = "$Revision: 1.10 $";
+static char RcsId[] = "$Revision: 5489 $";
 #endif
 
-/* $Id: cdeflate.c,v 1.10 1997/10/24 21:00:16 koziol Exp $ */
+/* $Id: cdeflate.c 5489 2010-10-23 06:52:48Z bmribler $ */
 
 /*
    FILE
@@ -48,15 +49,15 @@ static char RcsId[] = "$Revision: 1.10 $";
 /* #define TESTING */
 
 /* declaration of the functions provided in this module */
-PRIVATE int32 HCIcdeflate_init(accrec_t * access_rec);
+PRIVATE int32 HCIcdeflate_init(compinfo_t *info);
 
 /*--------------------------------------------------------------------------
  NAME
     HCIcdeflate_init -- Initialize a gzip 'deflate' compressed data element.
 
  USAGE
-    int32 HCIcdeflate_init(access_rec)
-    accrec_t *access_rec;   IN: the access record of the data element
+    int32 HCIcdeflate_init(info)
+    compinfo_t *info;           IN: special element information
 
  RETURNS
     Returns SUCCEED or FAIL
@@ -70,14 +71,11 @@ PRIVATE int32 HCIcdeflate_init(accrec_t * access_rec);
  REVISION LOG
 --------------------------------------------------------------------------*/
 PRIVATE int32
-HCIcdeflate_init(accrec_t * access_rec)
+HCIcdeflate_init(compinfo_t *info)
 {
     CONSTR(FUNC, "HCIcdeflate_init");
-    compinfo_t *info;       /* special element information */
     comp_coder_deflate_info_t *deflate_info;    /* ptr to deflate info */
 
-    info = (compinfo_t *) access_rec->special_info;
-
     if (Hseek(info->aid, 0, 0) == FAIL)  /* seek to beginning of element */
         HRETURN_ERROR(DFE_SEEKERROR, FAIL);
 
@@ -94,10 +92,6 @@ HCIcdeflate_init(accrec_t * access_rec)
     deflate_info->deflate_context.opaque=NULL;
     deflate_info->deflate_context.data_type=Z_BINARY;
 
-    /* Allocate compression I/O buffer */
-    if ((deflate_info->io_buf= HDmalloc(DEFLATE_BUF_SIZE)) == NULL)
-        HRETURN_ERROR(DFE_NOSPACE, FAIL);
-    
     return (SUCCEED);
 }   /* end HCIcdeflate_init() */
 
@@ -127,6 +121,7 @@ HCIcdeflate_decode(compinfo_t * info, int32 length, uint8 *buf)
 {
     CONSTR(FUNC, "HCIcdeflate_decode");
     comp_coder_deflate_info_t *deflate_info;    /* ptr to deflate info */
+    int zstat;              /* inflate status */
     int32 bytes_read;
 
     deflate_info = &(info->cinfo.coder_info.deflate_info);
@@ -136,17 +131,6 @@ HCIcdeflate_decode(compinfo_t * info, int32 length, uint8 *buf)
     deflate_info->deflate_context.avail_out=(uInt)length;
     while(deflate_info->deflate_context.avail_out>0)
       {
-#ifdef QAK
-int32 t1, t2;
-
-if(t1!=deflate_info->deflate_context.avail_out ||
-    t2!=deflate_info->deflate_context.avail_in)
-  {
-    printf("%s: avail_out=%d, avail_in=%d\n",FUNC,(int)deflate_info->deflate_context.avail_out,(int) deflate_info->deflate_context.avail_in);
-    t1=deflate_info->deflate_context.avail_out;
-    t2=deflate_info->deflate_context.avail_in;
-  }
-#endif
           /* Get more bytes from the file, if we've run out */
           if(deflate_info->deflate_context.avail_in==0)
             {
@@ -158,9 +142,22 @@ if(t1!=deflate_info->deflate_context.avail_out ||
                 deflate_info->deflate_context.avail_in=(uInt)file_bytes;
             } /* end if */
 
-          /* break out if we've reached the end of the compressed data somehow */
-          if(inflate(&(deflate_info->deflate_context),Z_NO_FLUSH)==Z_STREAM_END)
+          /* Read compressed data */
+          zstat=inflate(&(deflate_info->deflate_context),Z_NO_FLUSH);
+
+          /* break out if we've reached the end of the compressed data */
+          if (zstat == Z_STREAM_END)
               break;
+
+          /* break out if "inflate" returns reading errors */
+          else if (zstat == Z_VERSION_ERROR)
+          {
+              HRETURN_ERROR(DFE_COMPVERSION,FAIL);
+          }
+          else if (zstat <= Z_ERRNO && zstat > Z_VERSION_ERROR)
+          {
+              HRETURN_ERROR(DFE_READCOMP,FAIL);
+          }
       } /* end while */
     bytes_read=(int32)length-(int32)deflate_info->deflate_context.avail_out;
     deflate_info->offset+=bytes_read;
@@ -201,21 +198,24 @@ HCIcdeflate_encode(compinfo_t * info, int32 length, void * buf)
     /* Set up the deflation buffers to point to the user's buffer to empty */
     deflate_info->deflate_context.next_in=buf;
     deflate_info->deflate_context.avail_in=(uInt)length;
-    while(deflate_info->deflate_context.avail_in>0)
+    while(deflate_info->deflate_context.avail_in>0 || deflate_info->deflate_context.avail_out==0)
       {
           /* Write more bytes from the file, if we've filled our buffer */
           if(deflate_info->deflate_context.avail_out==0)
             {
                 if(deflate_info->deflate_context.next_out!=NULL)
+{
                     if(Hwrite(info->aid,DEFLATE_BUF_SIZE,deflate_info->io_buf)==FAIL)
                         HRETURN_ERROR(DFE_WRITEERROR,FAIL);
+}
                 deflate_info->deflate_context.next_out=deflate_info->io_buf;
                 deflate_info->deflate_context.avail_out=DEFLATE_BUF_SIZE;
             } /* end if */
 
           /* break out if we've reached the end of the compressed data somehow */
-          if(deflate(&(deflate_info->deflate_context),Z_NO_FLUSH)!=Z_OK)
+          if(deflate(&(deflate_info->deflate_context),Z_NO_FLUSH)!=Z_OK) {
               HRETURN_ERROR(DFE_CENCODE,FAIL);
+          }
       } /* end while */
     deflate_info->offset += length;    /* incr. abs. offset into the file */
 
@@ -252,7 +252,7 @@ HCIcdeflate_term(compinfo_t * info,uint32 acc_mode)
 
     /* set flag to indicate second stage of initialization is finished */
 
-    if(deflate_info->acc_init==1)
+    if(deflate_info->acc_init!=0)
       {
         if(acc_mode&DFACC_WRITE)
           { /* flush the "deflated" data to the file */
@@ -270,7 +270,7 @@ HCIcdeflate_term(compinfo_t * info,uint32 acc_mode)
                     } /* end if */
 
                     status=deflate(&(deflate_info->deflate_context),Z_FINISH);
-                } while(status==Z_OK);
+                } while(status==Z_OK || deflate_info->deflate_context.avail_out==0);
               if(status!=Z_STREAM_END)
                   HRETURN_ERROR(DFE_CENCODE,FAIL);
               if(deflate_info->deflate_context.avail_out<DEFLATE_BUF_SIZE)
@@ -289,8 +289,10 @@ HCIcdeflate_term(compinfo_t * info,uint32 acc_mode)
           } /* end else */
       } /* end if */
 
-    /* Get rid of the I/O buffer */
-    HDfree(deflate_info->io_buf);
+    /* Reset parameters */
+    deflate_info->offset = 0;   /* start at the beginning of the data */
+    deflate_info->acc_init = 0; /* second stage of initializing not performed */
+    deflate_info->acc_mode = 0; /* init access mode to illegal value */
 
     return (SUCCEED);
 }   /* end HCIcdeflate_term() */
@@ -337,12 +339,14 @@ HCIcdeflate_staccess(accrec_t * access_rec, int16 acc_mode)
     if (info->aid == FAIL)
         HRETURN_ERROR(DFE_DENIED, FAIL);
 #else /* OLD_WAY */
-    if (!(acc_mode&DFACC_WRITE))
+    if (!(acc_mode&DFACC_WRITE)) {
         info->aid = Hstartread(access_rec->file_id, DFTAG_COMPRESSED,
                                   info->comp_ref);
-    else
+      } /* end if */
+    else {
         info->aid = Hstartaccess(access_rec->file_id, DFTAG_COMPRESSED,
                                    info->comp_ref, DFACC_RDWR|DFACC_APPENDABLE);
+      } /* end else */
     if (info->aid == FAIL)
         HRETURN_ERROR(DFE_DENIED, FAIL);
 #endif /* OLD_WAY */
@@ -352,9 +356,13 @@ HCIcdeflate_staccess(accrec_t * access_rec, int16 acc_mode)
         HRETURN_ERROR(DFE_DENIED, FAIL);
 
     /* initialize the common deflate coding info */
-    if(HCIcdeflate_init(access_rec)==FAIL)
+    if(HCIcdeflate_init(info)==FAIL)
         HRETURN_ERROR(DFE_CODER,FAIL);
 
+    /* Allocate compression I/O buffer */
+    if ((deflate_info->io_buf= HDmalloc(DEFLATE_BUF_SIZE)) == NULL)
+        HRETURN_ERROR(DFE_NOSPACE, FAIL);
+    
     return (SUCCEED);
 }   /* end HCIcdeflate_staccess() */
 
@@ -415,7 +423,7 @@ HCIcdeflate_staccess2(accrec_t * access_rec, int16 acc_mode)
       } /* end else */
 
     /* set flag to indicate second stage of initialization is finished */
-    deflate_info->acc_init=1;
+    deflate_info->acc_init=acc_mode;
 
     return (SUCCEED);
 }   /* end HCIcdeflate_staccess2() */
@@ -510,7 +518,7 @@ HCPcdeflate_seek(accrec_t * access_rec, int32 offset, int origin)
     CONSTR(FUNC, "HCPcdeflate_seek");
     compinfo_t *info;           /* special element information */
     comp_coder_deflate_info_t *deflate_info;    /* ptr to gzip 'deflate' info */
-    uint8      *tmp_buf;        /* pointer to throw-away buffer */
+    uint8      tmp_buf[DEFLATE_TMP_BUF_SIZE];   /* temporary buffer */
 
     /* shut compiler up */
     origin = origin;
@@ -525,39 +533,56 @@ HCPcdeflate_seek(accrec_t * access_rec, int32 offset, int origin)
             HRETURN_ERROR(DFE_CINIT, FAIL);
       } /* end if */
 
-#ifdef QAK
-    /* Don't allow seeks when writing (currently) -QAK */
-    if (access_rec->access&DFACC_WRITE)
-        HRETURN_ERROR(DFE_UNSUPPORTED, FAIL);
-#endif /* QAK */
-
     if (offset < deflate_info->offset)
       {     /* need to seek from the beginning */
+#ifdef OLD_WAY
           /* Reset the decompression buffer */
           if (deflateReset(&(deflate_info->deflate_context))!=Z_OK)
               HRETURN_ERROR(DFE_CINIT, FAIL);
+#else /* OLD_WAY */
+        /* Terminate the previous method of access */
+        if (HCIcdeflate_term(info, deflate_info->acc_mode) == FAIL)
+            HRETURN_ERROR(DFE_CTERM, FAIL);
+
+        /* Restart access */
+        /* if (HCIcdeflate_staccess2(access_rec, deflate_info->acc_mode) == FAIL) */
+        if (HCIcdeflate_staccess2(access_rec, DFACC_READ) == FAIL)
+            HRETURN_ERROR(DFE_CINIT, FAIL);
+
+#endif /* OLD_WAY */
+
           /* Go back to the beginning of the data-stream */
           if(Hseek(info->aid,0,0)==FAIL)
               HRETURN_ERROR(DFE_SEEKERROR, FAIL);
       }     /* end if */
 
+#ifdef OLD_WAY
     if ((tmp_buf = (uint8 *) HDmalloc(DEFLATE_TMP_BUF_SIZE)) == NULL)     /* get tmp buffer */
         HRETURN_ERROR(DFE_NOSPACE, FAIL);
+#endif /* OLD_WAY */
 
-    while (deflate_info->offset + DEFLATE_TMP_BUF_SIZE < offset)    /* grab chunks */
+    while (deflate_info->offset + DEFLATE_TMP_BUF_SIZE < offset) {    /* grab chunks */
         if (HCIcdeflate_decode(info, DEFLATE_TMP_BUF_SIZE, tmp_buf) == FAIL)
           {
+#ifdef OLD_WAY
               HDfree(tmp_buf);
+#endif /* OLD_WAY */
               HRETURN_ERROR(DFE_CDECODE, FAIL)
           }     /* end if */
-    if (deflate_info->offset < offset)  /* grab the last chunk */
+      }     /* end if */
+    if (deflate_info->offset < offset) {  /* grab the last chunk */
         if (HCIcdeflate_decode(info, offset - deflate_info->offset, tmp_buf) == FAIL)
           {
+#ifdef OLD_WAY
               HDfree(tmp_buf);
+#endif /* OLD_WAY */
               HRETURN_ERROR(DFE_CDECODE, FAIL)
           }     /* end if */
+      }     /* end if */
 
+#ifdef OLD_WAY
     HDfree(tmp_buf);
+#endif /* OLD_WAY */
     return (SUCCEED);
 }   /* HCPcdeflate_seek() */
 
@@ -588,15 +613,28 @@ HCPcdeflate_read(accrec_t * access_rec, int32 length, void * data)
     CONSTR(FUNC, "HCPcdeflate_read");
     compinfo_t *info;           /* special element information */
     comp_coder_deflate_info_t *deflate_info;    /* ptr to gzip 'deflate' info */
+    uintn uninit;               /* Whether the interface was initialized */
 
     info = (compinfo_t *) access_rec->special_info;
     deflate_info = &(info->cinfo.coder_info.deflate_info);
 
     /* Check if second stage of initialization has been performed */
-    if(deflate_info->acc_init==0)
+    if(deflate_info->acc_init!=DFACC_READ)
       {
+        /* preserve the initialized state for later */
+        uninit=(deflate_info->acc_init!=0);
+
+        /* Terminate the previous method of access */
+        if (HCIcdeflate_term(info, deflate_info->acc_mode) == FAIL)
+            HRETURN_ERROR(DFE_CTERM, FAIL);
+
+        /* Restart access */
         if (HCIcdeflate_staccess2(access_rec, DFACC_READ) == FAIL)
             HRETURN_ERROR(DFE_CINIT, FAIL);
+
+        /* Go back to the beginning of the data-stream */
+        if(Hseek(info->aid,0,0)==FAIL)
+            HRETURN_ERROR(DFE_SEEKERROR, FAIL);
       } /* end if */
 
     if ((length=HCIcdeflate_decode(info, length, data)) == FAIL)
@@ -644,10 +682,19 @@ HCPcdeflate_write(accrec_t * access_rec, int32 length, const void * data)
         HRETURN_ERROR(DFE_UNSUPPORTED, FAIL);
 
     /* Check if second stage of initialization has been performed */
-    if(deflate_info->acc_init==0)
+    if(deflate_info->acc_init!=DFACC_WRITE)
       {
+        /* Terminate the previous method of access */
+        if (HCIcdeflate_term(info, deflate_info->acc_init) == FAIL)
+            HRETURN_ERROR(DFE_CTERM, FAIL);
+
+        /* Restart access */
         if (HCIcdeflate_staccess2(access_rec, DFACC_WRITE) == FAIL)
             HRETURN_ERROR(DFE_CINIT, FAIL);
+
+        /* Go back to the beginning of the data-stream */
+        if(Hseek(info->aid,0,0)==FAIL)
+            HRETURN_ERROR(DFE_SEEKERROR, FAIL);
       } /* end if */
 
     if ((length=HCIcdeflate_encode(info, length, (void *)data)) == FAIL)
@@ -737,6 +784,9 @@ HCPcdeflate_endaccess(accrec_t * access_rec)
     if (HCIcdeflate_term(info,deflate_info->acc_mode) == FAIL)
         HRETURN_ERROR(DFE_CTERM, FAIL);
 
+    /* Get rid of the I/O buffer */
+    HDfree(deflate_info->io_buf);
+
     /* close the compressed data AID */
     if (Hendaccess(info->aid) == FAIL)
         HRETURN_ERROR(DFE_CANTCLOSE, FAIL);
