@@ -1,20 +1,21 @@
-/****************************************************************************
- * NCSA HDF                                                                 *
- * Software Development Group                                               *
- * National Center for Supercomputing Applications                          *
- * University of Illinois at Urbana-Champaign                               *
- * 605 E. Springfield, Champaign IL 61820                                   *
- *                                                                          *
- * For conditions of distribution and use, see the accompanying             *
- * hdf/COPYING file.                                                        *
- *                                                                          *
- ****************************************************************************/
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * Copyright by The HDF Group.                                               *
+ * Copyright by the Board of Trustees of the University of Illinois.         *
+ * All rights reserved.                                                      *
+ *                                                                           *
+ * This file is part of HDF.  The full HDF copyright notice, including       *
+ * terms governing use, modification, and redistribution, is contained in    *
+ * the files COPYING and Copyright.html.  COPYING can be found at the root   *
+ * of the source code distribution tree; Copyright.html can be found at      *
+ * http://hdfgroup.org/products/hdf4/doc/Copyright.html.  If you do not have *
+ * access to either file, you may request a copy from help@hdfgroup.org.     *
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.14";
+static char RcsId[] = "@(#)$Revision: 6032 $";
 #endif
 
-/* cskphuff.c,v 1.14 1996/05/21 18:36:17 koziol Exp */
+/* $Id: cskphuff.c 6032 2014-01-17 18:13:52Z acheng $ */
 
 /*
    FILE
@@ -81,7 +82,7 @@ HCIcskphuff_splay(comp_coder_skphuff_info_t * skphuff_info, uint8 plain)
 {
     uintn       a, b;           /* children of nodes to semi-rotate */
     uint8       c, d;           /* pair of nodes to semi-rotate */
-    uintn       skip_num;       /* the tree we are splaying */
+    intn        skip_num;       /* the tree we are splaying */
     uintn       *lleft,        /* local copy of the left pointer */
                 *lright;       /* local copy of the right pointer */
     uint8       *lup;          /* local copy of the up pointer */
@@ -93,36 +94,36 @@ HCIcskphuff_splay(comp_coder_skphuff_info_t * skphuff_info, uint8 plain)
     lright=skphuff_info->right[skip_num];
     lup=skphuff_info->up[skip_num];
 
-    a = plain + SUCCMAX;    /* get the index for this source code in the up array */
+    a = (uintn)plain + SUCCMAX;    /* get the index for this source code in the up array */
     do
       {     /* walk up the tree, semi-rotating pairs */
           c = lup[a];    /* find the parent of the node to semi-rotate around */
           if (c != ROOT)
             {   /* a pair remain above this node */
-                d = lup[c];  /* get the grand-parent of the node to semi-rotate around */
-                b = lleft[d];
+                d = lup[(int)c];  /* get the grand-parent of the node to semi-rotate around */
+                b = lleft[(int)d];
 
 /* Exchange the children of the pair */
-                if (c == b)
+                if ((uintn)c == b)
                   {
-                      b = lright[d];
-                      lright[d] = a;
+                      b = lright[(int)d];
+                      lright[(int)d] = a;
                   }     /* end if */
                 else
-                    lleft[d] = a;
+                    lleft[(int)d] = a;
 
-                if (a == lleft[c])
-                    lleft[c] = b;
+                if (a == lleft[(int)c])
+                    lleft[(int)c] = b;
                 else
-                    lright[c] = b;
+                    lright[(int)c] = b;
 
                 lup[a] = d;
                 lup[b] = c;
-                a = d;
+                a = (uintn)d;
             }   /* end if */
           else
             {   /* handle odd node at end */
-                a = c;
+                a = (uintn)c;
             }   /* end else */
       }
     while (a != ROOT);
@@ -176,11 +177,11 @@ HCIcskphuff_init(accrec_t * access_rec, uintn alloc_buf)
     if(alloc_buf==TRUE)
       {
         /* allocate pointers to the compression buffers */
-        if ((skphuff_info->left = (uintn **) HDmalloc(sizeof(uintn *) * skphuff_info->skip_size)) == NULL)
+        if ((skphuff_info->left = (uintn **) HDmalloc(sizeof(uintn *) * (uintn)skphuff_info->skip_size)) == NULL)
                         HRETURN_ERROR(DFE_NOSPACE, FAIL);
-        if ((skphuff_info->right = (uintn **) HDmalloc(sizeof(uintn *) * skphuff_info->skip_size)) == NULL)
+        if ((skphuff_info->right = (uintn **) HDmalloc(sizeof(uintn *) * (uintn)skphuff_info->skip_size)) == NULL)
                         HRETURN_ERROR(DFE_NOSPACE, FAIL);
-        if ((skphuff_info->up = (uint8 **) HDmalloc(sizeof(uint8 *) * skphuff_info->skip_size)) == NULL)
+        if ((skphuff_info->up = (uint8 **) HDmalloc(sizeof(uint8 *) * (uintn)skphuff_info->skip_size)) == NULL)
                         HRETURN_ERROR(DFE_NOSPACE, FAIL);
 
 #ifdef TESTING
@@ -203,16 +204,13 @@ HCIcskphuff_init(accrec_t * access_rec, uintn alloc_buf)
 #endif /* TESTING */
     for (k = 0; k < skphuff_info->skip_size; k++)
       {
-#ifdef UNICOS
-#pragma novector
-#endif /* UNICOS */
           for (i = 0; i < TWICEMAX; i++)    /* initialize the up pointers to point to their parent in the tree */
-              skphuff_info->up[k][i] = i >> 1;
+              skphuff_info->up[k][i] = (uint8)(i >> 1);
 
           for (j = 0; j < SUCCMAX; j++)
             {   /* initialize the left & right pointers correctly */
-                skphuff_info->left[k][j] = j << 1;
-                skphuff_info->right[k][j] = (j << 1) + 1;
+                skphuff_info->left[k][j] = (uintn)(j << 1);
+                skphuff_info->right[k][j] = (uintn)((j << 1) + 1);
             }   /* end for */
       }     /* end for */
 
@@ -276,7 +274,7 @@ printf("bitcount=%d\n",++bitcount);
             }
           while (a <= SKPHUFF_MAX_CHAR);
 
-          plain = a - SUCCMAX;
+          plain = (uint8)(a - SUCCMAX);
           HCIcskphuff_splay(skphuff_info, plain);
           skphuff_info->skip_pos = (skphuff_info->skip_pos + 1) % skphuff_info->skip_size;
           *buf++ = plain;
@@ -295,7 +293,7 @@ printf("bitcount=%d\n",++bitcount);
     int32 HCIcskphuff_encode(info,length,buf)
     compinfo_t *info;   IN: the info about the compressed element
     int32 length;       IN: number of bytes to store from the buffer
-    uint8 *buf;         OUT: buffer to get the bytes from
+    const uint8 *buf;         OUT: buffer to get the bytes from
 
  RETURNS
     Returns SUCCEED or FAIL
@@ -309,7 +307,7 @@ printf("bitcount=%d\n",++bitcount);
  REVISION LOG
 --------------------------------------------------------------------------*/
 PRIVATE int32
-HCIcskphuff_encode(compinfo_t * info, int32 length, uint8 *buf)
+HCIcskphuff_encode(compinfo_t * info, int32 length, const uint8 *buf)
 {
     CONSTR(FUNC, "HCIcskphuff_encode");
     comp_coder_skphuff_info_t *skphuff_info;    /* ptr to skipping Huffman info */
@@ -330,7 +328,7 @@ HCIcskphuff_encode(compinfo_t * info, int32 length, uint8 *buf)
     orig_length = length;   /* save this for later */
     while (length > 0)
       {     /* encode until we stored all the bytes */
-          a = *buf + SUCCMAX;   /* find position in the up array */
+          a = (uintn)*buf + SUCCMAX;   /* find position in the up array */
 #ifdef OLD_WAY
           do
             {   /* walk up the tree, pushing bits */
@@ -356,7 +354,7 @@ HCIcskphuff_encode(compinfo_t * info, int32 length, uint8 *buf)
           do
             {   /* walk up the tree, pushing bits */
                 last_node=a; /* keep track of the current node */
-                a = skphuff_info->up[skphuff_info->skip_pos][a]; /* move the current node up one */
+                a = (uintn)skphuff_info->up[skphuff_info->skip_pos][a]; /* move the current node up one */
                 if(skphuff_info->right[skphuff_info->skip_pos][a] == last_node)
                     output_bits[stack_ptr]|=bit_mask; /* push a 1 if this is the right node */
                 bit_mask<<=1;   /* rotate bit mask over */
@@ -374,7 +372,7 @@ HCIcskphuff_encode(compinfo_t * info, int32 length, uint8 *buf)
           do {   /* output the bits we have */
                 if(bit_count[stack_ptr]>0)
                   {
-                    if(Hbitwrite(info->aid,bit_count[stack_ptr],output_bits[stack_ptr]) !=bit_count[stack_ptr])
+                    if(Hbitwrite(info->aid,(intn)bit_count[stack_ptr],output_bits[stack_ptr]) !=(intn)bit_count[stack_ptr])
                         HRETURN_ERROR(DFE_CENCODE, FAIL);
                   } /* end if */
                 stack_ptr--;
@@ -476,8 +474,16 @@ HCIcskphuff_staccess(accrec_t * access_rec, int16 acc_mode)
         info->aid = Hstartbitread(access_rec->file_id, DFTAG_COMPRESSED,
                                   info->comp_ref);
     else
+#ifdef OLD_WAY
         info->aid = Hstartbitwrite(access_rec->file_id, DFTAG_COMPRESSED,
                                    info->comp_ref, info->length);
+#else /* OLD_WAY */
+      {
+        info->aid = Hstartbitwrite(access_rec->file_id, DFTAG_COMPRESSED,
+                                   info->comp_ref, 0);
+        Hbitappendable(info->aid);
+      } /* end else */
+#endif /* OLD_WAY */
 
 #ifdef TESTING
     printf("HCIcskphuff_staccess(): after bitio calls\n");
@@ -608,13 +614,13 @@ HCPcskphuff_seek(accrec_t * access_rec, int32 offset, int origin)
         if (HCIcskphuff_decode(info, TMP_BUF_SIZE, tmp_buf) == FAIL)
           {
               HDfree(tmp_buf);
-              HRETURN_ERROR(DFE_CDECODE, FAIL);
+              HRETURN_ERROR(DFE_CDECODE, FAIL)
           }     /* end if */
     if (skphuff_info->offset < offset)  /* grab the last chunk */
         if (HCIcskphuff_decode(info, offset - skphuff_info->offset, tmp_buf) == FAIL)
           {
               HDfree(tmp_buf);
-              HRETURN_ERROR(DFE_CDECODE, FAIL);
+              HRETURN_ERROR(DFE_CDECODE, FAIL)
           }     /* end if */
 
     HDfree(tmp_buf);
@@ -629,7 +635,7 @@ HCPcskphuff_seek(accrec_t * access_rec, int32 offset, int origin)
     int32 HCPcskphuff_read(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to read
-    VOIDP data;             OUT: the buffer to place the bytes read
+    void * data;             OUT: the buffer to place the bytes read
 
  RETURNS
     Returns the number of bytes read or FAIL
@@ -643,7 +649,7 @@ HCPcskphuff_seek(accrec_t * access_rec, int32 offset, int origin)
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPcskphuff_read(accrec_t * access_rec, int32 length, VOIDP data)
+HCPcskphuff_read(accrec_t * access_rec, int32 length, void * data)
 {
     CONSTR(FUNC, "HCPcskphuff_read");
     compinfo_t *info;           /* special element information */
@@ -664,7 +670,7 @@ HCPcskphuff_read(accrec_t * access_rec, int32 length, VOIDP data)
     int32 HCPcskphuff_write(access_rec,length,data)
     accrec_t *access_rec;   IN: the access record of the data element
     int32 length;           IN: the number of bytes to write
-    VOIDP data;             IN: the buffer to retrieve the bytes written
+    void * data;             IN: the buffer to retrieve the bytes written
 
  RETURNS
     Returns the number of bytes written or FAIL
@@ -678,7 +684,7 @@ HCPcskphuff_read(accrec_t * access_rec, int32 length, VOIDP data)
  REVISION LOG
 --------------------------------------------------------------------------*/
 int32
-HCPcskphuff_write(accrec_t * access_rec, int32 length, const VOIDP data)
+HCPcskphuff_write(accrec_t * access_rec, int32 length, const void * data)
 {
     CONSTR(FUNC, "HCPcskphuff_write");
     compinfo_t *info;           /* special element information */
