@@ -1,3 +1,16 @@
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * Copyright by The HDF Group.                                               *
+ * Copyright by the Board of Trustees of the University of Illinois.         *
+ * All rights reserved.                                                      *
+ *                                                                           *
+ * This file is part of HDF.  The full HDF copyright notice, including       *
+ * terms governing use, modification, and redistribution, is contained in    *
+ * the files COPYING and Copyright.html.  COPYING can be found at the root   *
+ * of the source code distribution tree; Copyright.html can be found at      *
+ * http://hdfgroup.org/products/hdf4/doc/Copyright.html.  If you do not have *
+ * access to either file, you may request a copy from help@hdfgroup.org.     *
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
 /*
  * Name:
  *      fp2hdf
@@ -160,6 +173,11 @@
 #include <string.h>
 #include <ctype.h>
 
+#ifdef _WIN32
+#include <sys/stat.h>
+#include <fcntl.h>
+#endif
+
 /*
  * global macros
  */
@@ -355,11 +373,16 @@ main(int argc, char *argv[])
     int         token;
     int         state = 0;
 
+    
     const char *err1 = "Invalid number of arguments:  %d.\n";
     const char *err2 = "Error in state table.\n";
     const char *err3 = "No output file given.\n";
     const char *err4 = "Program aborted.\n";
 
+#ifdef _WIN32
+	_fmode = _O_BINARY;
+#endif
+
     /*
      * set 'stdout' and 'stderr' to line-buffering mode
      */
@@ -377,6 +400,9 @@ main(int argc, char *argv[])
       }
 
     opt.to_image = FALSE;   /* default: no image */
+    opt.to_float = FALSE;   /* default: make float if no image */
+			    /* Set FALSE here.  Will be set TRUE */
+			    /* after confirming image option is not set.  */
     opt.ctm = EXPAND;   /* default: pixel replication */
     opt.hres = 0;   /* default: no expansion values */
     opt.vres = 0;
@@ -442,7 +468,7 @@ main(int argc, char *argv[])
                     opt.mean = TRUE;
                     break;
                 case 16:    /* mean value */
-                    opt.meanval = atof(argv[i]);
+                    opt.meanval = (float32)atof(argv[i]);
                     break;
                 case ERR:   /* command syntax error */
                 default:
@@ -472,11 +498,11 @@ main(int argc, char *argv[])
     if (process(&opt))
         goto err;
 
-    exit(0);
+    return(0);
 
   err:
     (void) fprintf(stderr, err4);
-    exit(1);
+    return(1);
 }
 
 /*
@@ -699,13 +725,14 @@ gdimen(char *infile, struct Input *inp, FILE *strm)
 static int
 gfloat(char *infile, FILE * strm, float32 *fp32, struct Input *in)
 {
-    float64     fp64;
+    float64     fp64=0.0;
 
     const char *err1 = "Unable to get 'float' value from file: %s.\n";
 
     if (in->is_text == TRUE)
       {
-          if (fscanf(strm, "%e", fp32) != 1)
+
+		if (fscanf(strm, "%e", fp32) != 1)
             {
                 (void) fprintf(stderr, err1, infile);
                 goto err;
@@ -713,6 +740,7 @@ gfloat(char *infile, FILE * strm, float32 *fp32, struct Input *in)
       }
     else if (in->is_fp32 == TRUE)
       {
+		
           if (fread((char *) fp32, sizeof(float32), 1, strm) != 1)
             {
                 (void) fprintf(stderr, err1, infile);
@@ -998,7 +1026,7 @@ gscale(char *infile, struct Input *in, FILE *strm, int *is_scale)
 static int
 gtoken(char *s)
 {
-    int         len;
+    size_t      len;
     int         token;
 
     const char *err1 = "Illegal argument: %s.\n";
@@ -1093,8 +1121,12 @@ gtype(char *infile, struct Input *in, FILE **strm)
                 (void) fprintf(stderr, err2, infile);
                 goto err;
             }
-          if (!HDmemcmp("TEXT", buf, 4) || !HDmemcmp("text", buf, 4))
+          if (!HDmemcmp("TEXT", buf, 4) || !HDmemcmp("text", buf, 4)) {
+#ifdef _WIN32
+			  _fmode = _O_TEXT;
+#endif
               in->is_text = TRUE;
+		  }
           else
             {
                 rewind(*strm);
@@ -1113,6 +1145,7 @@ gtype(char *infile, struct Input *in, FILE **strm)
                       (void) fprintf(stderr, err3, infile);
                       goto err;
                   }
+		
             }
       }
 
@@ -1130,8 +1163,7 @@ gtype(char *infile, struct Input *in, FILE **strm)
  *      Print a helpful summary of command usage and features.
  */
 void
-help(name)
-char       *name;
+help(char *name)
 {
     (void) printf("Name:\n");
     (void) printf("\t%s\n\n", name);
@@ -1327,20 +1359,19 @@ indexes(float32 *scale, int dim, int *idx, int res)
     /*
      * determine the midpoints between scale values
      */
-    if ((midpt = (float32 *) HDmalloc((unsigned int) (dim * sizeof(float32))))
-        == NULL)
+    if ((midpt = (float32 *) HDmalloc((size_t) dim * sizeof(float32))) == NULL)
       {
           (void) fprintf(stderr, err1);
           goto err;
       }
     for (i = 0; i < dim - 1; i++)
-        midpt[i] = (scale[i] + scale[i + 1]) * 0.5;
+        midpt[i] = (scale[i] + scale[i + 1]) * (float32)0.5;
     midpt[dim - 1] = scale[dim - 1] + (scale[dim - 1] - midpt[dim - 2]);
 
     /*
      * determine the distance between pixel locations
      */
-    delta = (scale[dim - 1] - scale[0]) / (res - 1.0);
+    delta = (scale[dim - 1] - scale[0]) / (float32)(res - 1);
 
     /*
      * compute indexes, keeping the index the same until the location
@@ -1400,8 +1431,8 @@ interp(struct Input *in, struct Raster *im)
     float32     loc;
     float32     range;
     float32     ratio;
-    float32     hrange, vrange, drange = 0;
-    float32     hdelta, vdelta, ddelta = 0;
+    float32     hrange, vrange, drange = (float32)0.0;
+    float32     hdelta, vdelta, ddelta = (float32)0.0;
     float32     t1, t2, t3, t4, t5, t6;
     float32    *hratio, *vratio, *dratio = NULL;
     float32    *pt[8];
@@ -1413,7 +1444,7 @@ interp(struct Input *in, struct Raster *im)
      * determine the range of pixel locations
      */
     range = in->max - in->min;
-    ratio = 237.9 / range;
+    ratio = (float32)237.9 / range;
     hrange = in->hscale[in->dims[0] - 1] - in->hscale[0];
     vrange = in->vscale[in->dims[1] - 1] - in->vscale[0];
     if (in->rank == 3)
@@ -1422,16 +1453,15 @@ interp(struct Input *in, struct Raster *im)
     /*
      * determine the distance between pixel locations
      */
-    hdelta = hrange / (im->hres - 1);
-    vdelta = vrange / (im->vres - 1);
+    hdelta = hrange / (float32)(im->hres - 1);
+    vdelta = vrange / (float32)(im->vres - 1);
     if (in->rank == 3)
-        ddelta = drange / (im->dres - 1);
+        ddelta = drange / (float32)(im->dres - 1);
 
     /*
      * allocate dynamic memory for the interpolation ratio buffers
      */
-    if ((hratio = (float32 *) HDmalloc((unsigned int) im->hres *
-                                         sizeof(float32))) == NULL)
+    if ((hratio = (float32 *) HDmalloc((size_t) im->hres * sizeof(float32))) == NULL)
       {
           (void) fprintf(stderr, err1);
           goto err;
@@ -1486,7 +1516,7 @@ interp(struct Input *in, struct Raster *im)
       {
           loc = hdelta * (float) i + in->hscale[0];
           hinc[i] = 0;
-          while ((j < (in->dims[0] - 2)) && ((hrange > 0) ?
+          while ((j < (in->dims[0] - 2)) && ((hrange > (float32)0.0) ?
                      (in->hscale[j + 1] < loc) : (in->hscale[j + 1] > loc)))
             {
                 hinc[i] += 1;
@@ -1497,7 +1527,7 @@ interp(struct Input *in, struct Raster *im)
     for (i = 0, j = 0, voff[0] = 0; i < im->vres; i++)
       {
           loc = vdelta * (float) i + in->vscale[0];
-          while ((j < (in->dims[1] - 2)) && ((vrange > 0) ?
+          while ((j < (in->dims[1] - 2)) && ((vrange > (float32)0.0) ?
                      (in->vscale[j + 1] < loc) : (in->vscale[j + 1] > loc)))
             {
                 voff[i] += 1;
@@ -1511,7 +1541,7 @@ interp(struct Input *in, struct Raster *im)
           for (i = 0, j = 0, doff[0] = 0; i < im->dres; i++)
             {
                 loc = ddelta * (float) i + in->dscale[0];
-                while ((j < (in->dims[2] - 2)) && ((drange > 0) ?
+                while ((j < (in->dims[2] - 2)) && ((drange > (float32)0.0) ?
                      (in->dscale[j + 1] < loc) : (in->dscale[j + 1] > loc)))
                   {
                       doff[i] += 1;
@@ -1547,7 +1577,7 @@ interp(struct Input *in, struct Raster *im)
                           pix = in->max;    /* clip (bug fix) */
                       if (pix < in->min)
                           pix = in->min;    /* ditto */
-                      *ip++ = (ratio * (pix - in->min)) + 1.5;
+                      *ip++ = (unsigned char)((ratio * (pix - in->min)) + (float32)1.5);
                   }
             }
       }
@@ -1585,7 +1615,7 @@ interp(struct Input *in, struct Raster *im)
                                 pix = in->max;  /* clip (bug fix) */
                             if (pix < in->min)
                                 pix = in->min;  /* ditto */
-                            *ip++ = (ratio * (pix - in->min)) + 1.5;
+                            *ip++ = (unsigned char)((ratio * (pix - in->min)) + (float32)1.5);
                         }
                   }
             }
@@ -1655,14 +1685,12 @@ isnum(char *s)
  *      the user-specified mean value.
  */
 void
-mean(in, opt)
-struct Input *in;
-struct Options *opt;
+mean(struct Input *in,struct Options * opt)
 {
     float32     delta, delta_max, delta_min;
 
-    delta_max = fabs(in->max - opt->meanval);
-    delta_min = fabs(opt->meanval - in->min);
+    delta_max = (float32)fabs((double)(in->max - opt->meanval));
+    delta_min = (float32)fabs((double)(opt->meanval - in->min));
     delta = (delta_max > delta_min) ? delta_max : delta_min;
 
     in->max = opt->meanval + delta;
@@ -1693,7 +1721,7 @@ palette(char *palfile)
     /*
      * extract a palette from an HDF file
      */
-    if (!Hishdf(palfile))
+    if (Hishdf(palfile))
       {
           if (DFPgetpal(palfile, pal))
             {
@@ -1781,7 +1809,7 @@ pixrep(struct Input *in, struct Raster *im)
     dp = (float32 *) in->data;
     ip = im->image;
     range = in->max - in->min;
-    ratio = 237.9 / range;
+    ratio = (float32)237.9 / range;
 
     /*
      * determine the scale indexes of the horizontal pixel locations
@@ -1850,7 +1878,7 @@ pixrep(struct Input *in, struct Raster *im)
                       if (vidx[j] > ovidx)
                         {
                             for (i = 0; i < in->dims[0]; i++)
-                                pix[i] = (ratio * (*dp++ - in->min)) + 1.5;
+                                pix[i] = (unsigned char )((ratio * (*dp++ - in->min)) + (float32)1.5);
                             for (i = 0; i < im->hres; i++)
                                 *ip++ = pix[hidx[i]];
                             /*
@@ -1973,22 +2001,22 @@ process(struct Options *opt)
           /*
            * get the scale for each axis
            */
-          if ((in.hscale = (float32 *) HDmalloc((unsigned int)
-                             ((in.dims[0] + 1) * sizeof(float32)))) == NULL)
+          if ((in.hscale = (float32 *) HDmalloc((size_t)
+                             (in.dims[0] + 1) * sizeof(float32))) == NULL)
             {
                 (void) fprintf(stderr, err2);
                 goto err;
             }
-          if ((in.vscale = (float32 *) HDmalloc((unsigned int)
-                             ((in.dims[1] + 1) * sizeof(float32)))) == NULL)
+          if ((in.vscale = (float32 *) HDmalloc((size_t)
+                             (in.dims[1] + 1) * sizeof(float32))) == NULL)
             {
                 (void) fprintf(stderr, err2);
                 goto err;
             }
           if (in.rank == 3)
             {
-                if ((in.dscale = (float32 *) HDmalloc((unsigned int)
-                             ((in.dims[2] + 1) * sizeof(float32)))) == NULL)
+                if ((in.dscale = (float32 *) HDmalloc((size_t)
+                             (in.dims[2] + 1) * sizeof(float32))) == NULL)
                   {
                       (void) fprintf(stderr, err2);
                       goto err;
@@ -2001,8 +2029,8 @@ process(struct Options *opt)
            * get the input data
            */
           len = in.dims[0] * in.dims[1] * in.dims[2];
-          if ((in.data = (VOIDP) HDmalloc((unsigned int) (len *
-                                                 sizeof(float32)))) == NULL)
+          if ((in.data = (VOIDP) HDmalloc((size_t) len *
+                                                 sizeof(float32))) == NULL)
             {
                 (void) fprintf(stderr, err2);
                 goto err;
@@ -2230,8 +2258,7 @@ process(struct Options *opt)
  *      Print a summary of command usage.
  */
 void
-usage(name)
-char       *name;
+usage(char *name)
 {
     (void) fprintf(stderr, "\nUsage:\t%s -h[elp], OR\n", name);
     (void) fprintf(stderr, "\t%s <infile> [<infile>...] ", name);
