@@ -1,20 +1,21 @@
-/****************************************************************************
- * NCSA HDF                                                                 *
- * Software Development Group                                               *
- * National Center for Supercomputing Applications                          *
- * University of Illinois at Urbana-Champaign                               *
- * 605 E. Springfield, Champaign IL 61820                                   *
- *                                                                          *
- * For conditions of distribution and use, see the accompanying             *
- * hdf/COPYING file.                                                        *
- *                                                                          *
- ****************************************************************************/
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * Copyright by The HDF Group.                                               *
+ * Copyright by the Board of Trustees of the University of Illinois.         *
+ * All rights reserved.                                                      *
+ *                                                                           *
+ * This file is part of HDF.  The full HDF copyright notice, including       *
+ * terms governing use, modification, and redistribution, is contained in    *
+ * the files COPYING and Copyright.html.  COPYING can be found at the root   *
+ * of the source code distribution tree; Copyright.html can be found at      *
+ * http://hdfgroup.org/products/hdf4/doc/Copyright.html.  If you do not have *
+ * access to either file, you may request a copy from help@hdfgroup.org.     *
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.30";
+static char RcsId[] = "@(#)$Revision: 4937 $";
 #endif
 
-/* he_cntrl.c,v 1.30 1996/05/16 18:43:23 koziol Exp */
+/* $Id: he_cntrl.c 4937 2007-09-10 16:36:16Z pvn $ */
 
 /* ------ he-cntrl.c ------
    This file contains much of the contol mechanisms for HDFed
@@ -47,7 +48,7 @@ HEif(HE_CMD * cmd)
         return FAIL;
 
     /* execute the sub list only is the predicates are satisfied */
-    if (satPred(currDesc(), pred))
+    if (satPred(currDesc, pred))
       {
           /* go through sub-list until an end is encountered */
           for (cmdTail = cmd->sub;
@@ -93,7 +94,7 @@ HEselect(HE_CMD * cmd)
 
     /* step through all elements */
     for (he_currDesc = 0; he_currDesc < he_numDesc; he_currDesc++)
-        if (currTag() != DFTAG_NULL && satPred(currDesc(), pred))
+        if (currTag != DFTAG_NULL && satPred(currDesc, pred)) {
             for (cmdTail = cmd->sub; HDstrcmp(cmdTail->argv[0], "end");
                  cmdTail = cmdTail->next)
                 if (cmdTail->func)
@@ -104,6 +105,7 @@ HEselect(HE_CMD * cmd)
                               cmdTail->argv[0]);
                       he_status = FAIL;
                   }
+        }
 
     /* restore he_currDesc */
     he_currDesc = t_currDesc;
@@ -311,7 +313,7 @@ HEdump(HE_CMD * cmd)
               unkArg(cmd->argv[i]);
               return HE_FAIL;
           }
-    return dump(length, offset, format, raw);
+    return HEIdump(length, offset, format, raw);
 }
 
 #define MAX_LINE 60
@@ -320,7 +322,7 @@ HEdump(HE_CMD * cmd)
  * Run 'od' on a segment of the current data element
  */
 int
-dump(int32 length, int offset, char *format, int raw_flag)
+HEIdump(int32 length, int offset, char *format, int raw_flag)
 {
     int32       eltLength;
     int32 i;
@@ -476,7 +478,7 @@ dump(int32 length, int offset, char *format, int raw_flag)
                   intn        sizeintn;
 
                   sizeintn = sizeof(intn);
-                  idata = (intn *) HDmalloc(length / 4 * sizeintn);
+                  idata = (intn *) HDmalloc((size_t)(length / 4 * sizeintn));
                   DFKconvert((VOIDP) (data + offset), (VOIDP) idata, DFNT_NINT32 | raw_flag,
                              length / 4, DFACC_READ, 0, 0);
                   printf("%8d: ", offset);
@@ -500,7 +502,7 @@ dump(int32 length, int offset, char *format, int raw_flag)
                   intn        sizeintn;
 
                   sizeintn = sizeof(intn);
-                  idata = (intn *) HDmalloc(length / 4 * sizeintn);
+                  idata = (intn *) HDmalloc((size_t)(length / 4 * sizeintn));
                   DFKconvert((VOIDP) (data + offset), (VOIDP) idata, DFNT_NINT32 | raw_flag,
                              length / 4, DFACC_READ, 0, 0);
                   printf("%8d: ", offset);
@@ -667,7 +669,7 @@ info(int all, int longout, int group, int label)
           return HE_OK;
       }
 
-    if (!group || (!isGrp(currTag()) && !all))
+    if (!group || (!isGrp(currTag) && !all))
       {
           if (all)
             {
@@ -709,7 +711,7 @@ info(int all, int longout, int group, int label)
                       fprintf(stderr, "There is no group in this file.\n");
                       return HE_FAIL;
                   }
-                start = end = currGrpNo();
+                start = end = currGrpNo;
             }
           for (i = start; i <= end; i++)
             {
@@ -735,11 +737,12 @@ info(int all, int longout, int group, int label)
             {
                 puts("\n**These do not belong to any group:");
                 for (i = 0; i < he_numDesc; i++)
-                    if (!mark[i])
+                    if (!mark[i]) {
                         if (he_desc[i].tag == DFTAG_NULL)
                             empty++;
                         else
                             infoDesc(i, longout, label);
+                    }
 
                 if (empty > 0)
                     printf("Empty (tag %d) : %d slots.\n", DFTAG_NULL, empty);
@@ -898,8 +901,8 @@ int         he_batch = NO;
 int         he_nestLevel = 0;
 
 /* prompt is actually "he"<he_prompt><space> */
-char       *he_prompt = ">";
-char        he_nestChar = '>';
+const char  *he_prompt = ">";
+#define he_nestChar '>'
 
 /* table to associate command to the function --
    add additional functions anywhere in the table BEFORE the
@@ -1010,7 +1013,7 @@ findFunc(char *fword)
 {
     unsigned    len;
     int         found = -1;
-    int i;
+    uintn i;
 
     len = HDstrlen((const char *) fword);
 
@@ -1022,7 +1025,7 @@ findFunc(char *fword)
                   return he_funcTab[i].func;
 
               if (found < 0)
-                  found = i;
+                  found = (int)i;
               else
                 {
                     fprintf(stderr, "Ambiguous command: %s.\n", fword);
@@ -1111,7 +1114,7 @@ getLine(char *p)
                       ch = getchar();
                   break;
               case HE_SEPARATOR:
-                  if (!isspace(*(p - 1)))
+                  if (!isspace((int)*(p - 1)))
                       *p++ = SPACE;
                   *p++ = HE_SEPARATOR;
                   ch = SPACE;   /* Ensure next is a space */
@@ -1136,7 +1139,7 @@ nextWord(char **p)
     unsigned    len;
 
     q = *p;
-    while (*q && isspace(*q))
+    while (*q && isspace((int)*q))
         q++;
     if (!(*q))
       {
@@ -1145,7 +1148,7 @@ nextWord(char **p)
       }
 
     s = q;
-    while (*s && !isspace(*s))
+    while (*s && !isspace((int)*s))
         s++;
     len = (unsigned) (s - q);
 
@@ -1154,7 +1157,7 @@ nextWord(char **p)
     word[len] = '\0';
 
     *p = s;
-    while (**p && (isspace(**p)))
+    while (**p && (isspace((int)**p)))
         (*p)++;
 
     return word;
@@ -1200,7 +1203,7 @@ parseCmd(char **p)
                word = nextWord(p), cmdTail->argc++)
               cmdTail->argv[cmdTail->argc] = word;
 
-          while (**p && (isspace(**p) || (**p == ';')))
+          while (**p && (isspace((int)**p) || (**p == ';')))
               (*p)++;
       }
     return cmd;
@@ -1493,7 +1496,7 @@ he_keyTab[] =
 int
 findKey(char *word)
 {
-    int i;
+    uintn i;
     unsigned    len;
     int         found = -1;
 
@@ -1506,7 +1509,7 @@ findKey(char *word)
               if (HDstrlen(he_keyTab[i].str) == len)
                   return he_keyTab[i].key;
               if (found < 0)
-                  found = i;
+                  found = (int)i;
               else
                 {
                     fprintf(stderr, "Ambiguous: %s.\n", word);
@@ -1689,11 +1692,11 @@ nextToken(char **p)
 
     s = *p;
 
-    if (isalnum(**p))
-        while (isalnum(*s))
+    if (isalnum((int)**p))
+        while (isalnum((int)*s))
             s++;
     else
-        while (*s && !isalnum(*s))
+        while (*s && !isalnum((int)*s))
             s++;
 
     q = tok = (char *) HDmalloc((s - (*p)) + 1);
