@@ -1,20 +1,21 @@
-/****************************************************************************
- * NCSA HDF                                                                 *
- * Software Development Group                                               *
- * National Center for Supercomputing Applications                          *
- * University of Illinois at Urbana-Champaign                               *
- * 605 E. Springfield, Champaign IL 61820                                   *
- *                                                                          *
- * For conditions of distribution and use, see the accompanying             *
- * hdf/COPYING file.                                                        *
- *                                                                          *
- ****************************************************************************/
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * Copyright by The HDF Group.                                               *
+ * Copyright by the Board of Trustees of the University of Illinois.         *
+ * All rights reserved.                                                      *
+ *                                                                           *
+ * This file is part of HDF.  The full HDF copyright notice, including       *
+ * terms governing use, modification, and redistribution, is contained in    *
+ * the files COPYING and Copyright.html.  COPYING can be found at the root   *
+ * of the source code distribution tree; Copyright.html can be found at      *
+ * http://hdfgroup.org/products/hdf4/doc/Copyright.html.  If you do not have *
+ * access to either file, you may request a copy from help@hdfgroup.org.     *
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifdef RCSID
-static char RcsId[] = "@(#)1.35";
+static char RcsId[] = "@(#)$Revision: 6036 $";
 #endif
 
-/* he_main.c,v 1.35 1996/06/24 21:42:09 acheng Exp */
+/* $Id: he_main.c 6036 2014-01-20 17:28:01Z acheng $ */
 
 /******************************************************************************
  * he - HDF editor
@@ -66,14 +67,12 @@ static char RcsId[] = "@(#)1.35";
  *****************************************************************************/
 /* ------ he.c ------- main() main HDF interfacing routines */
 
-#if defined __MWERKS__
-#include <console.h>
-#endif
-
 #include "he.h"
 
 #include <stdio.h>
+#ifndef _WIN32
 #include <unistd.h>
+#endif
 
 /* the return status of last command executed */
 int         he_status = HE_OK;
@@ -92,10 +91,6 @@ main(int argc, char *argv[])
     int i;
     char       *fileName = NULL;
 
-#if defined __MWERKS__
-    argc = ccommand(&argv);
-#endif
-
     for (i = 1; i < argc; i++)
       {
           if (argv[i][0] == '-')
@@ -152,7 +147,7 @@ main(int argc, char *argv[])
     cmdLoop();
 
     if (fileOpen())
-        close(YES);     /* close with keep */
+        closeFile(YES);     /* close with keep */
     quit(0);
     return 0;
 }
@@ -213,35 +208,6 @@ getElement(int desc, char **pdata)
     return length;
 }
 
-/* The function is not called.
-
-   int od(char *format, char *file)
-   {
-   fork a child and let the child run od. Use vfork for VMS.
-
-   if (fork() == 0)
-
-   {
-   this is the child
-
-   if (execl("/bin/od", "od", format, file, 0) == -1)
-   fprintf(stderr, "Error while executing od.\n");
-
-   return control to the parent
-   exit(0);
-   }
-
-   the parent waits for the child to die
-
-   wait(0);
-
-   this is a bug because it always returns OK, will expand this as
-   soon as the status return mechanism from wait is understood
-   return HE_OK;
-
-   }
- */
-
 /* the tmp directory, currently set for unix */
 #define TDIR "/tmp/"
 
@@ -255,7 +221,7 @@ getTmpName(char **pname)
     (void) sprintf(s, "%she%d.%d", TDIR, (int)getpid(), count);
     count++;
 
-    length = HDstrlen(s);
+    length = (int)HDstrlen(s);
     if (length <= 0)
         return FAIL;
 
@@ -275,7 +241,7 @@ writeToFile(char *file, char *data, int32 length)
     if (fd == NULL)
         return FAIL;
 
-    written = fwrite(data, sizeof(char), (size_t) length, fd);
+    written = (int)fwrite(data, sizeof(char), (size_t) length, fd);
     if (written != length)
       {
           fprintf(stderr, "Error in write.\n");
@@ -289,11 +255,7 @@ writeToFile(char *file, char *data, int32 length)
 int
 removeFile(char *file)
 {
-#ifndef VMS
-    return unlink(file);
-#else
     return remove((const char *) file);
-#endif
 }
 
 /* is a file currently opened */
@@ -304,7 +266,7 @@ fileOpen(void)
 }
 
 char       *
-backupName(char *file)
+backupName(const char *file)
 {
     return catStr(file, "$hdfed$");
 }
@@ -339,8 +301,8 @@ copyFile(char *from, char *to)
           return FAIL;
       }
     /* copy the contents from hdf file to backup file */
-    while ((num_read = fread(buf, 1, HE_BUF_SZ, fp)) > 0)
-        fwrite(buf, 1, num_read, bfp);
+    while ((num_read = (int)fread(buf, 1, HE_BUF_SZ, fp)) > 0)
+        fwrite(buf, 1, (size_t)num_read, bfp);
 
     fclose(fp);
     fclose(bfp);
@@ -351,7 +313,7 @@ copyFile(char *from, char *to)
 int
 updateDesc(void)
 {
-    uint32      fid;
+    int32      fid;
     int32       groupID;
     int32       aid, status;
     int i, j;
@@ -468,7 +430,6 @@ getR8(int xdim, int ydim, char *image, char *pal, int compress)
     FILE       *fp;
     int32       length;
     char       *buf;
-    int32       ret;
 
     if (!fileOpen())
       {
@@ -488,14 +449,14 @@ getR8(int xdim, int ydim, char *image, char *pal, int compress)
           fprintf(stderr, "Error opening image file: %s.\n", image);
           return FAIL;
       }
-    if ((ret = fread(buf, xdim, ydim, fp)) < ydim)
+    if (fread(buf, (size_t)xdim, (size_t)ydim, fp) < (size_t)ydim)
       {
           fprintf(stderr, "Error reading image file: %s.\n", image);
           return FAIL;
       }
     fclose(fp);
 
-    if ((ret = DFR8addimage(he_file, buf, (int32) xdim, (int32) ydim, (uint16) compress)) < 0)
+    if (DFR8addimage(he_file, buf, (int32) xdim, (int32) ydim, (uint16) compress) < 0)
       {
           HEprint(stderr, 0);
           return FAIL;
@@ -516,16 +477,15 @@ setPal(char *pal)
     char        palette[HE_PALETTE_SZ];
     char *p;
     int i;
-    int         ret;
 
     if ((fp = fopen(pal, "r")) == NULL)
       {
           fprintf(stderr, "Error opening palette file: %s.\n", pal);
           return FAIL;
       }
-    if ((ret = fread(reds, 1, HE_COLOR_SZ, fp)) < HE_COLOR_SZ ||
-        (ret = fread(greens, 1, HE_COLOR_SZ, fp)) < HE_COLOR_SZ ||
-        (ret = fread(blues, 1, HE_COLOR_SZ, fp)) < HE_COLOR_SZ)
+    if (fread(reds, 1, HE_COLOR_SZ, fp) < HE_COLOR_SZ ||
+        fread(greens, 1, HE_COLOR_SZ, fp) < HE_COLOR_SZ ||
+        fread(blues, 1, HE_COLOR_SZ, fp) < HE_COLOR_SZ)
       {
           fprintf(stderr, "Error reading palette file: %s.\n", pal);
           return FAIL;
@@ -623,7 +583,7 @@ getCurrRig(int32 *pXdim, int32 *pYdim, char **pPalette, char **pRaster)
         *pPalette = (char *) HDmalloc(HE_PALETTE_SZ);
     else
         *pPalette = (char *) NULL;
-    *pRaster = (char *) HDmalloc((*pXdim) * (*pYdim));
+    *pRaster = (char *) HDmalloc((size_t)(*pXdim) * (size_t)(*pYdim));
 
     if (DFR8getimage(he_file, (unsigned char *) *pRaster, *pXdim, *pYdim,
                      (unsigned char *) *pPalette) == FAIL)
@@ -729,9 +689,9 @@ putElement(char *file, uint16 tag, uint16 ref, char *data, int32 len)
     int32       ret;
     int32       fid;
 
-    if ((fid = Hopen(file, DFACC_READ | DFACC_WRITE, 0)) == (uint32) NULL)
+    if ((fid = Hopen(file, DFACC_READ | DFACC_WRITE, 0)) == FAIL)
         /* a little tricky here */
-        if (HEvalue(0) != DFE_FNF || (fid = Hopen(file, DFACC_ALL, 0)) == (uint32) NULL)
+        if (HEvalue(0) != DFE_FNF || (fid = Hopen(file, DFACC_ALL, 0)) == FAIL)
           {
               HEprint(stderr, 0);
               return FAIL;
@@ -757,7 +717,7 @@ writeGrp(char *file)
 
     getNewRef(file, &ref);
 
-    grp = currGrpNo();
+    grp = currGrpNo;
     gid = DFdisetup(he_grp[grp].size);
     for (i = 0; i < he_grp[grp].size; i++)
       {
@@ -769,12 +729,12 @@ writeGrp(char *file)
       }
     /* do the group now */
 
-    if ((fid = Hopen(file, DFACC_READ | DFACC_WRITE, 0)) == (uint32) NULL)
+    if ((fid = Hopen(file, DFACC_READ | DFACC_WRITE, 0)) == FAIL)
       {
           HEprint(stderr, 0);
           return FAIL;
       }
-    if ((ret = DFdiwrite(fid, gid, currTag(), ref)) < 0)
+    if ((ret = DFdiwrite(fid, gid, currTag, ref)) < 0)
       {
           HEprint(stderr, 0);
           return ret;
@@ -787,9 +747,9 @@ getNewRef(char *file, uint16 *pRef)
 {
     int32       fid;
 
-    if ((fid = Hopen(file, DFACC_READ | DFACC_WRITE, 0)) == (uint32) NULL)
+    if ((fid = Hopen(file, DFACC_READ | DFACC_WRITE, 0)) == FAIL)
         /* a little tricky here */
-        if (HEvalue(0) != DFE_FNF || (fid = Hopen(file, DFACC_ALL, 0)) == (uint32) NULL)
+        if (HEvalue(0) != DFE_FNF || (fid = Hopen(file, DFACC_ALL, 0)) == FAIL)
           {
               HEprint(stderr, 0);
               return FAIL;
@@ -847,7 +807,7 @@ writeAnnot(char *file, uint16 tag, uint16 ref)
 }
 
 int32
-getAnn(int ann, int16 tag, int16 ref, char **pBuf)
+getAnn(int ann, uint16 tag, uint16 ref, char **pBuf)
 {
     int32       len;
 
@@ -856,7 +816,7 @@ getAnn(int ann, int16 tag, int16 ref, char **pBuf)
           len = DFANgetlablen(he_file, tag, ref);
           if (len > 0)
             {
-                *pBuf = (char *) HDmalloc(len + 1);
+                *pBuf = (char *) HDmalloc((size_t)(len + 1));
                 DFANgetlabel(he_file, tag, ref, *pBuf, len + 1);
             }
           else
@@ -867,7 +827,7 @@ getAnn(int ann, int16 tag, int16 ref, char **pBuf)
           len = DFANgetdesclen(he_file, tag, ref);
           if (len > 0)
             {
-                *pBuf = (char *) HDmalloc(len);
+                *pBuf = (char *) HDmalloc((size_t)len);
                 DFANgetdesc(he_file, tag, ref, *pBuf, len);
             }
           else
@@ -915,7 +875,7 @@ readFromFile(char *file, char **pBuf)
           if (*pBuf == NULL)
               return FAIL;
 
-          num_read = fread((*pBuf) + soFar, 1, HE_BUF_SZ, fp);
+          num_read = (int32)fread((*pBuf) + soFar, 1, HE_BUF_SZ, fp);
       }
     *pBuf = (char *) HDrealloc(*pBuf, soFar + 1);
     (*pBuf)[soFar] = '\0';
@@ -1094,7 +1054,7 @@ findOpt(char *word)
 {
     unsigned    len;
     int         found = -1;
-    int i;
+    uintn i;
 
     len = HDstrlen(word);
 
@@ -1106,7 +1066,7 @@ findOpt(char *word)
                   return he_optTab[i].key;
 
               if (found < 0)
-                  found = i;
+                  found = (int)i;
               else
                   return HE_AMBIG;
           }
@@ -1118,7 +1078,7 @@ findOpt(char *word)
 }
 
 char       *
-catStr(char *s, char *s1)
+catStr(const char *s, const char *s1)
 {
     char       *t;
 
@@ -1303,7 +1263,7 @@ fillTemplate(char **template, char **pout, char *s, char templateChar)
     /* count length of template to replace */
     for (templateLen = 0; **template == templateChar;
          (*template)++, templateLen++) ;
-    sLen = HDstrlen(s);
+    sLen = (int)HDstrlen(s);
 
     /* fill with zero's if the space reserved in template is
        longer than the length of s */
